@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2019-2020 Scoopta
+ *  Copyright (C) 2019-2025 Scoopta
  *  This file is part of Wofi
  *  Wofi is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -82,6 +82,7 @@ static void print_usage(char** argv) {
 	printf("--term\t\t\t-t\tSpecifies the terminal to use when running in a term\n");
 	printf("--password\t\t-P\tRuns in password mode\n");
 	printf("--exec-search\t\t-e\tMakes enter always use the search contents not the first result\n");
+	printf("--no-custom-entry\t-E\tOnly entries will be submitted to modes, never the content of the search box\n");
 	printf("--hide-scroll\t\t-b\tHides the scroll bars\n");
 	printf("--matching\t\t-M\tSets the matching method, default is contains\n");
 	printf("--insensitive\t\t-i\tAllows case insensitive searching\n");
@@ -97,6 +98,7 @@ static void print_usage(char** argv) {
 	printf("--search\t\t-Q\tSearch for something immediately on open\n");
 	printf("--monitor\t\t-o\tSets the monitor to open on\n");
 	printf("--pre-display-cmd\t-r\tRuns command for the displayed entries, without changing the output. %%s for the real string\n");
+  	printf("--hide-search\t\t-j\tHides the search bar\n");
 	exit(0);
 }
 
@@ -341,6 +343,12 @@ int main(int argc, char** argv) {
 			.flag = NULL,
 			.val = 'e'
 		},
+		{
+			.name = "no-custom-entry",
+			.has_arg = no_argument,
+			.flag = NULL,
+			.val = 'E'
+		},
 		{
 			.name = "hide-scroll",
 			.has_arg = no_argument,
@@ -431,6 +439,12 @@ int main(int argc, char** argv) {
 			.flag = NULL,
 			.val = 'r'
 		},
+		{
+			.name = "hide-search",
+			.has_arg = no_argument,
+			.flag = NULL,
+			.val = 'j'
+		},
 		{
 			.name = NULL,
 			.has_arg = 0,
@@ -455,6 +469,7 @@ int main(int argc, char** argv) {
 	char* terminal = NULL;
 	char* password_char = "false";
 	char* exec_search = NULL;
+	char* no_custom_entry = NULL;
 	char* hide_scroll = NULL;
 	char* matching = NULL;
 	char* insensitive = NULL;
@@ -468,13 +483,14 @@ int main(int argc, char** argv) {
 	char* search = NULL;
 	char* monitor = NULL;
 	char* pre_display_cmd = NULL;
+  	char* hide_search = NULL;
 
 	struct wl_list options;
 	wl_list_init(&options);
 	struct option_node* node;
 
 	int opt;
-	while((opt = getopt_long(argc, argv, "hfc:s:C:dS:W:H:p:x:y:nImk:t:P::ebM:iqvl:aD:L:w:O:GQ:o:r:", opts, NULL)) != -1) {
+	while((opt = getopt_long(argc, argv, "hfc:s:C:dS:W:H:p:x:y:nImk:t:P::eEbM:iqvl:aD:L:w:O:GQ:o:r:j", opts, NULL)) != -1) {
 		switch(opt) {
 		case 'h':
 			print_usage(argv);
@@ -538,6 +554,9 @@ int main(int argc, char** argv) {
 		case 'e':
 			exec_search = "true";
 			break;
+		case 'E':
+			no_custom_entry = "true";
+			break;
 		case 'b':
 			hide_scroll = "true";
 			break;
@@ -586,6 +605,9 @@ int main(int argc, char** argv) {
 		case 'r':
 			pre_display_cmd = optarg;
 			break;
+		case 'j':
+			hide_search = "true";
+			break;
 		}
 	}
 
@@ -742,6 +764,9 @@ int main(int argc, char** argv) {
 	if(exec_search != NULL) {
 		map_put(config, "exec_search", exec_search);
 	}
+	if(no_custom_entry != NULL) {
+		map_put(config, "no_custom_entry", no_custom_entry);
+	}
 	if(hide_scroll != NULL) {
 		map_put(config, "hide_scroll", hide_scroll);
 	}
@@ -778,6 +803,9 @@ int main(int argc, char** argv) {
 	if(pre_display_cmd != NULL) {
 		map_put(config, "pre_display_cmd", pre_display_cmd);
 	}
+  	if(hide_search != NULL) {
+		map_put(config, "hide_search", hide_search);
+	}
 
 	struct sigaction sigact = {0};
 	sigact.sa_handler = sig;
