@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2019-2024 Scoopta
+ *  Copyright (C) 2019-2025 Scoopta
  *  This file is part of Wofi
  *  Wofi is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -79,6 +79,7 @@ static bool mod_ctrl;
 static char* terminal;
 static GtkOrientation outer_orientation;
 static bool exec_search;
+static bool no_custom_entry;
 static struct map* modes;
 static enum matching_mode matching;
 static bool insensitive;
@@ -109,6 +110,7 @@ static char* copy_exec = NULL;
 static char* pre_display_cmd = NULL;
 static bool pre_display_exec = false;
 static bool single_click = false;
+static bool hide_search = false;
 static GdkModifierType shift_mask = GDK_SHIFT_MASK;
 static GdkModifierType ctrl_mask = GDK_CONTROL_MASK;
 static GdkModifierType alt_mask = GDK_MOD1_MASK;
@@ -572,7 +574,9 @@ static void expand(GtkExpander* expander, gpointer data) {
 static void update_surface_size(void) {
 	if(lines > 0) {
 		height = max_height * lines;
-		height += 5;
+	}
+	if(width == 0 || height == 0) {
+		return;
 	}
 	if(shell != NULL) {
 		zwlr_layer_surface_v1_set_size(wlr_surface, width, height);
@@ -581,17 +585,7 @@ static void update_surface_size(void) {
 	}
 
 	gtk_window_resize(GTK_WINDOW(window), width, height);
-	GtkAllocation alloc;
-	gtk_widget_get_allocated_size(entry, &alloc, NULL);
-	if(outer_orientation == GTK_ORIENTATION_HORIZONTAL) {
-		if(alloc.width > 0) {
-			gtk_widget_set_size_request(scroll, width - alloc.width, height);
-		}
-	} else {
-		if(alloc.height > 0) {
-			gtk_widget_set_size_request(scroll, width, height - alloc.height);
-		}
-	}
+	gtk_widget_set_size_request(scroll, width, height);
 }
 
 static void widget_allocate(GtkWidget* widget, GdkRectangle* allocation, gpointer data) {
@@ -926,7 +920,16 @@ struct wl_list* wofi_read_cache(struct mode* mode) {
 				smallest_node = node;
 			}
 		}
-		char* tmp = strdup(strchr(smallest_node->line, ' ') + 1);
+
+		char* space = strchr(smallest_node->line, ' ');
+
+		if(space == NULL) {
+			free(smallest_node->line);
+			wl_list_remove(&smallest_node->link);
+			continue;
+		}
+
+		char* tmp = strdup(space + 1);
 		free(smallest_node->line);
 		smallest_node->line = tmp;
 		wl_list_remove(&smallest_node->link);
@@ -968,6 +971,10 @@ bool wofi_allow_markup(void) {
 	return allow_markup;
 }
 
+bool wofi_no_custom_entry(void) {
+	return no_custom_entry;
+}
+
 uint64_t wofi_get_image_size(void) {
 	return image_size;
 }
@@ -1121,8 +1128,8 @@ static gint do_sort(GtkFlowBoxChild* child1, GtkFlowBoxChild* child2, gpointer d
 	return sort_for_matching_mode(text1, text2, fallback, matching, filter, insensitive);
 }
 
-static void select_first(void) {
-	GtkFlowBoxChild* child = gtk_flow_box_get_child_at_index(GTK_FLOW_BOX(inner_box), 0);
+static void select_idx(gint idx) {
+	GtkFlowBoxChild* child = gtk_flow_box_get_child_at_index(GTK_FLOW_BOX(inner_box), idx);
 	gtk_widget_grab_focus(GTK_WIDGET(child));
 	gtk_flow_box_select_child(GTK_FLOW_BOX(inner_box), GTK_FLOW_BOX_CHILD(child));
 }
@@ -1175,7 +1182,7 @@ static void move_down(void) {
 	user_moved = true;
 	if(outer_orientation == GTK_ORIENTATION_VERTICAL) {
 		if(gtk_widget_has_focus(entry) || gtk_widget_has_focus(scroll)) {
-			select_first();
+			select_idx(1);
 			return;
 		}
 	}
@@ -1191,7 +1198,7 @@ static void move_right(void) {
 	user_moved = true;
 	if(outer_orientation == GTK_ORIENTATION_HORIZONTAL) {
 		if(gtk_widget_has_focus(entry) || gtk_widget_has_focus(scroll)) {
-			select_first();
+			select_idx(1);
 			return;
 		}
 	}
@@ -1201,14 +1208,14 @@ static void move_right(void) {
 static void move_forward(void) {
 	user_moved = true;
 	if(gtk_widget_has_focus(entry) || gtk_widget_has_focus(scroll)) {
-		select_first();
+		select_idx(1);
 		return;
 	}
 
 	gtk_widget_child_focus(window, GTK_DIR_TAB_FORWARD);
 
 	if(gtk_widget_has_focus(entry)) {
-		select_first();
+		select_idx(0);
 	}
 }
 
@@ -1251,7 +1258,9 @@ static void do_expand(void) {
 }
 
 static void do_hide_search(void) {
-	gtk_widget_set_visible(entry, !gtk_widget_get_visible(entry));
+	bool visible = gtk_widget_get_visible(entry);
+	gtk_widget_set_visible(entry, !visible);
+	gtk_widget_set_sensitive(entry, !visible);
 	update_surface_size();
 }
 
@@ -1497,6 +1506,9 @@ static gboolean focus(GtkWidget* widget, GdkEvent* event, gpointer data) {
 		gtk_widget_set_state_flags(widget, GTK_STATE_FLAG_FOCUSED, TRUE);
 	} else {
 		gtk_widget_set_state_flags(widget, GTK_STATE_FLAG_NORMAL, TRUE);
+		if (data != NULL && strcmp(config_get(data, "close_on_focus_loss", "false"), "true") == 0) {
+        	gtk_widget_destroy(widget);
+		}
 	}
 	return FALSE;
 }
@@ -1571,7 +1583,7 @@ static void* load_mode(char* _mode, char* name, struct mode* mode_ptr, struct ma
 		arg_count = get_arg_count();
 	}
 
-	if(mode == NULL && no_entry != NULL && no_entry()) {
+	if(mode == NULL && !no_custom_entry && no_entry != NULL && no_entry()) {
 		mode = mode_ptr->name;
 	}
 
@@ -1747,6 +1759,14 @@ static gboolean do_percent_size_first(gpointer data){
 	return G_SOURCE_REMOVE;
 }
 
+static gboolean hide_search_first(gpointer data) {
+	(void) data;
+	gtk_widget_set_visible(entry, !hide_search);
+	gtk_widget_set_sensitive(entry, !hide_search);
+	update_surface_size();
+	return G_SOURCE_REMOVE;
+}
+
 void wofi_init(struct map* _config) {
 	config = _config;
 	char* width_str = config_get(config, "width", "50%");
@@ -1782,8 +1802,9 @@ void wofi_init(struct map* _config) {
 	terminal = map_get(config, "term");
 	char* password_char = map_get(config, "password_char");
 	exec_search = strcmp(config_get(config, "exec_search", "false"), "true") == 0;
+	no_custom_entry = strcmp(config_get(config, "no_custom_entry", "false"), "true") == 0;
 	bool hide_scroll = strcmp(config_get(config, "hide_scroll", "false"), "true") == 0;
-	matching = config_get_mnemonic(config, "matching", "contains", 3, "contains", "multi-contains", "fuzzy");
+	matching = config_get_mnemonic(config, "matching", "contains", 4, "contains", "strict-contains", "multi-contains", "fuzzy");
 	insensitive = strcmp(config_get(config, "insensitive", "false"), "true") == 0;
 	parse_search = strcmp(config_get(config, "parse_search", "false"), "true") == 0;
 	location = config_get_mnemonic(config, "location", "center", 18,
@@ -1796,7 +1817,7 @@ void wofi_init(struct map* _config) {
 	sort_order = config_get_mnemonic(config, "sort_order", "default", 2, "default", "alphabetical");
 	line_wrap = config_get_mnemonic(config, "line_wrap", "off", 4, "off", "word", "char", "word_char") - 1;
 	bool global_coords = strcmp(config_get(config, "global_coords", "false"), "true") == 0;
-	bool hide_search = strcmp(config_get(config, "hide_search", "false"), "true") == 0;
+	hide_search = strcmp(config_get(config, "hide_search", "false"), "true") == 0;
 	char* search = map_get(config, "search");
 	dynamic_lines = strcmp(config_get(config, "dynamic_lines", "false"), "true") == 0;
 	char* monitor = map_get(config, "monitor");
@@ -1820,7 +1841,7 @@ void wofi_init(struct map* _config) {
 		char* key_default;
 
 		key_default = "Up";
-		char* key_up = (i == 0) ? "Up" : config_get(config, "key_up", key_default);
+		char* key_up = (i == 0) ? key_default : config_get(config, "key_up", key_default);
 		key_default = "Down";
 		char* key_down = (i == 0) ? key_default : config_get(config, "key_down", key_default);
 		key_default = "Left";
@@ -1997,13 +2018,19 @@ void wofi_init(struct map* _config) {
 	outer_box = gtk_box_new(outer_orientation, 0);
 	gtk_widget_set_name(outer_box, "outer-box");
 	gtk_container_add(GTK_CONTAINER(window), outer_box);
-	entry = gtk_search_entry_new();
+
+	bool use_search_box = strcmp(config_get(config, "use_search_box", "true"), "true") == 0;
+	if(use_search_box) {
+		entry = gtk_search_entry_new();
+	} else {
+		entry = gtk_entry_new();
+	}
+
 	g_signal_connect(entry, "size-allocate", G_CALLBACK(widget_allocate), NULL);
 
 	gtk_widget_set_name(entry, "input");
 	gtk_entry_set_placeholder_text(GTK_ENTRY(entry), prompt);
 	gtk_container_add(GTK_CONTAINER(outer_box), entry);
-	gtk_widget_set_child_visible(entry, !hide_search);
 
 	if(search != NULL) {
 		gtk_entry_set_text(GTK_ENTRY(entry), search);
@@ -2044,7 +2071,7 @@ void wofi_init(struct map* _config) {
 	g_signal_connect(entry, "activate", G_CALLBACK(activate_search), NULL);
 	g_signal_connect(window, "key-press-event", G_CALLBACK(key_press), NULL);
 	g_signal_connect(window, "focus-in-event", G_CALLBACK(focus), NULL);
-	g_signal_connect(window, "focus-out-event", G_CALLBACK(focus), NULL);
+	g_signal_connect(window, "focus-out-event", G_CALLBACK(focus), config);
 	g_signal_connect(entry, "focus-in-event", G_CALLBACK(focus_entry), NULL);
 	g_signal_connect(entry, "focus-out-event", G_CALLBACK(focus_entry), NULL);
 	g_signal_connect(window, "destroy", G_CALLBACK(do_exit), NULL);
@@ -2063,6 +2090,8 @@ void wofi_init(struct map* _config) {
 		gdk_threads_add_timeout(25, do_percent_size_first, geo_str);
 	}
 
+	gdk_threads_add_timeout(5, hide_search_first, NULL);
+
 	wl_list_init(&mode_list);
 
 	pthread_create(&mode_thread, NULL, start_mode_thread, mode);
